# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx.schemas.base import get_config

OPTIONS = """
charset utf-8;
"""


def get_charset_config():
    return get_config(options=OPTIONS,
                      indent=0)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.listen import get_listen_config
from polyaxon_nginx.schemas.locations import get_locations_config
from polyaxon_nginx.schemas.logging import get_logging_config
from polyaxon_nginx.schemas.plugins import get_dns_config, get_plugins_location_config
from polyaxon_nginx.schemas.redirect import get_redirect_config
from polyaxon_nginx.schemas.ssl import get_ssl_config
from polyaxon_nginx.schemas.timeout import get_timeout_config
from polyaxon_nginx.schemas.ws import get_ws_location_config


class TestSchemas(TestCase):

    def test_timeout(self):
        expected = """
send_timeout 200;
keepalive_timeout 200;
uwsgi_read_timeout 200;
uwsgi_send_timeout 200;
client_header_timeout 200;
proxy_read_timeout 200;
"""
        settings.TIMEOUT = 200
        assert get_timeout_config() == expected

    def test_listen(self):
        expected = """
listen 80;
"""
        settings.SSL_ENABLED = False
        assert get_listen_config() == expected

        expected = """
listen 443 ssl;
ssl on;
"""
        settings.SSL_ENABLED = True
        assert get_listen_config() == expected

    def test_ssl(self):
        expected = """
# SSL
ssl_session_timeout 1d;
ssl_session_cache shared:SSL:50m;
ssl_session_tickets off;

# modern configuration
ssl_protocols TLSv1.2;
ssl_ciphers ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256;
ssl_prefer_server_ciphers on;

# OCSP Stapling
ssl_stapling on;
ssl_stapling_verify on;
resolver 1.1.1.1 1.0.0.1 8.8.8.8 8.8.4.4 208.67.222.222 208.67.220.220 valid=60s;
resolver_timeout 2s;

ssl_certificate      /etc/ssl/polyaxon/polyaxon.com.crt;
ssl_certificate_key  /etc/ssl/polyaxon/polyaxon.com.key;
"""  # noqa
        assert get_ssl_config() == expected

        expected = """
# SSL
ssl_session_timeout 1d;
ssl_session_cache shared:SSL:50m;
ssl_session_tickets off;

# modern configuration
ssl_protocols TLSv1.2;
ssl_ciphers ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256;
ssl_prefer_server_ciphers on;

# OCSP Stapling
ssl_stapling on;
ssl_stapling_verify on;
resolver 1.1.1.1 1.0.0.1 8.8.8.8 8.8.4.4 208.67.222.222 208.67.220.220 valid=60s;
resolver_timeout 2s;

ssl_certificate      /foo/polyaxon.com.crt;
ssl_certificate_key  /foo/polyaxon.com.key;
"""  # noqa
        settings.SSL_PATH = '/foo'
        assert get_ssl_config() == expected

    def test_redirect_config(self):
        expected = """
server {
    listen 80;
    return 301 https://$host$request_uri;
}
"""
        settings.SSL_ENABLED = False
        assert get_redirect_config() == ''
        settings.SSL_ENABLED = True
        assert get_redirect_config() == expected

    def test_logging(self):
        expected = """
error_log /polyaxon/logs/error.log warn;
"""
        settings.LOGGING_LEVEL = 'warn'
        assert get_logging_config() == expected

    def test_locations(self):
        expected = """
location /static/ {
    alias /polyaxon/static/;
    autoindex on;
    expires                   30d;
    add_header                Cache-Control private;
}


location /tmp/ {
    alias                     /tmp/;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}


location /repos/ {
    root                      /repos;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}


location /data/ {
    root                      /data;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}


location /outputs/ {
    root                      /outputs;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}
"""
        assert get_locations_config() == expected

    def test_no_plugins(self):
        assert get_plugins_location_config() == []

    def test_plugins(self):
        settings.NGINX_PLUGINS = {'tensorboard': {'port': 6006}, 'notebook': {'port': 8888}}
        assert len(get_plugins_location_config()) == 2

    def test_plugins_dns_resolver(self):
        expected = """
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}
"""  # noqa
        settings.DNS_USE_RESOLVER = False
        assert '\n'.join(get_plugins_location_config()) == expected

        expected = """
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}
"""  # noqa
        settings.DNS_PREFIX = 'kube-dns.kube-system'
        settings.DNS_USE_RESOLVER = True
        settings.DNS_CUSTOM_CLUSTER = 'new-dns'
        assert '\n'.join(get_plugins_location_config()) == expected

    def test_plugins_dns_backend(self):
        settings.DNS_USE_RESOLVER = True
        expected = """
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}
"""  # noqa
        settings.DNS_CUSTOM_CLUSTER = 'cluster.local'
        assert get_dns_config() == 'kube-dns.kube-system.svc.cluster.local'
        assert '\n'.join(get_plugins_location_config()) == expected

        expected = """
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.kube-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}
"""  # noqa
        settings.DNS_CUSTOM_CLUSTER = 'new-dns'
        assert get_dns_config() == 'kube-dns.kube-system.svc.new-dns'
        assert '\n'.join(get_plugins_location_config()) == expected

    def test_plugins_dns_prefix(self):
        settings.DNS_USE_RESOLVER = True
        expected = """
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    resolver coredns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    resolver coredns.kube-system.svc.cluster.local valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}
"""  # noqa
        settings.DNS_PREFIX = 'coredns.kube-system'
        settings.DNS_CUSTOM_CLUSTER = 'cluster.local'
        assert get_dns_config() == 'coredns.kube-system.svc.cluster.local'
        assert '\n'.join(get_plugins_location_config()) == expected

        expected = """
location ~ /tensorboard/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.new-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/tensorboard/proxy/([-_.:\w]+)/(.*) /tensorboard/proxy/$1/$2 break;
    proxy_pass http://$1:6006;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}


location ~ /notebook/proxy/([-_.:\w]+)/(.*) {
    resolver kube-dns.new-system.svc.new-dns valid=5s;
    rewrite_log on;
    rewrite ^/notebook/proxy/([-_.:\w]+)/(.*) /notebook/proxy/$1/$2 break;
    proxy_pass http://$1:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}
"""  # noqa
        settings.DNS_PREFIX = 'kube-dns.new-system'
        settings.DNS_CUSTOM_CLUSTER = 'new-dns'
        assert get_dns_config() == 'kube-dns.new-system.svc.new-dns'
        assert '\n'.join(get_plugins_location_config()) == expected

    def test_ws_location_config(self):
        # settings.NGINX_PLUGINS = {'tensorboard': {'port': 6006}, 'notebook': {'port': 8888}}
        expected = """
location /ws/ {
    proxy_pass http://localhost:1337;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    internal;
}
"""
        assert get_ws_location_config() == expected

        settings.WS_PORT = 8888
        expected = """
location /ws/ {
    proxy_pass http://localhost:8888;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    internal;
}
"""
        assert get_ws_location_config() == expected



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx.schemas.base import get_config

OPTIONS = """
error_page 500 502 503 504 /50x.html;
"""


def get_error_page_config():
    return get_config(options=OPTIONS,
                      indent=0)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.base import get_config

SSL_OPTIONS = """
# SSL
ssl_session_timeout 1d;
ssl_session_cache shared:SSL:50m;
ssl_session_tickets off;

# modern configuration
ssl_protocols TLSv1.2;
ssl_ciphers ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256;
ssl_prefer_server_ciphers on;

# OCSP Stapling
ssl_stapling on;
ssl_stapling_verify on;
resolver 1.1.1.1 1.0.0.1 8.8.8.8 8.8.4.4 208.67.222.222 208.67.220.220 valid=60s;
resolver_timeout 2s;

ssl_certificate      {ssl_path}/polyaxon.com.crt;
ssl_certificate_key  {ssl_path}/polyaxon.com.key;
"""  # noqa


def get_ssl_config():
    return get_config(options=SSL_OPTIONS,
                      indent=0,
                      ssl_path=settings.SSL_PATH)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx.schemas.polyaxon import get_polyaxon_config
from polyaxon_nginx.schemas.redirect import get_redirect_config


def write_to_conf_file(name, content, path=None):
    with open('{}/{}.conf'.format(path or '.', name), 'w') as f:
        f.write(content)


def generate_nginx_conf(path=None):
    write_to_conf_file('polyaxon.base', get_polyaxon_config(), path)
    write_to_conf_file('polyaxon.redirect', get_redirect_config(), path)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from polyaxon_nginx import settings


class TestSettings(TestCase):
    def test_default_values(self):
        assert settings.NGINX_PLUGINS == {}
        assert settings.DNS_USE_RESOLVER is False
        assert settings.DNS_CUSTOM_CLUSTER == 'cluster.local'
        assert settings.DNS_BACKEND == 'kube-dns'
        assert settings.DNS_PREFIX is None
        assert settings.NAMESPACE is None
        assert settings.LOGGING_LEVEL == 'warn'
        assert settings.TIMEOUT == 600
        assert settings.SSL_PATH == '/etc/ssl/polyaxon'
        assert settings.INDENT_CHAR == ' '
        assert settings.INDENT_WIDTH == 4
        assert settings.SSL_ENABLED is False
        assert settings.REPOS_PATH == '/repos'
        assert settings.DATA_PATH == '/data'
        assert settings.OUTPUTS_PATH == '/outputs'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx.schemas.base import get_config

OPTIONS = """
upstream polyaxon {{
  server unix:/polyaxon/web/polyaxon.sock;
}}

server {{
    include polyaxon/polyaxon.base.conf;
}}
"""


def get_main_config():
    return get_config(options=OPTIONS,
                      indent=0)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx.schemas.base import get_config

UWSGI_OPTIONS = """
location / {{
    include     /etc/nginx/uwsgi_params;
    uwsgi_pass  polyaxon;
    uwsgi_param Host				$host;
    uwsgi_param X-Real-IP			$remote_addr;
    uwsgi_param X-Forwarded-For		$proxy_add_x_forwarded_for;
    uwsgi_param X-Forwarded-Proto	$http_x_forwarded_proto;
}}
"""


def get_uwsgi_config():
    return get_config(options=UWSGI_OPTIONS,
                      indent=0)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx.schemas.base import get_config

OPTIONS = """
gzip                        on;
gzip_disable                "msie6";
gzip_types                  *;
"""


def get_gzip_config():
    return get_config(options=OPTIONS,
                      indent=0)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from polyaxon_nginx.schemas.polyaxon import get_polyaxon_config


class TestPolyaxonSchemas(TestCase):

    def test_polyaxon_config(self):
        expected = """
listen 80;


error_log /polyaxon/logs/error.log warn;


gzip                        on;
gzip_disable                "msie6";
gzip_types                  *;


charset utf-8;


client_max_body_size        4G;
client_body_buffer_size     50m;
client_body_in_file_only clean;
sendfile on;


send_timeout 600;
keepalive_timeout 600;
uwsgi_read_timeout 600;
uwsgi_send_timeout 600;
client_header_timeout 600;
proxy_read_timeout 600;


location / {
    include     /etc/nginx/uwsgi_params;
    uwsgi_pass  polyaxon;
    uwsgi_param Host				$host;
    uwsgi_param X-Real-IP			$remote_addr;
    uwsgi_param X-Forwarded-For		$proxy_add_x_forwarded_for;
    uwsgi_param X-Forwarded-Proto	$http_x_forwarded_proto;
}


error_page 500 502 503 504 /50x.html;


location /static/ {
    alias /polyaxon/static/;
    autoindex on;
    expires                   30d;
    add_header                Cache-Control private;
}


location /tmp/ {
    alias                     /tmp/;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}


location /repos/ {
    root                      /repos;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}


location /data/ {
    root                      /data;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}


location /outputs/ {
    root                      /outputs;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}


location /ws/ {
    proxy_pass http://localhost:1337;
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    internal;
}
"""
        assert get_polyaxon_config() == expected



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from unittest import TestCase

from polyaxon_nginx.schemas.main import get_main_config


class TestBaseSchemas(TestCase):

    def test_base_config(self):
        expected = """
upstream polyaxon {
  server unix:/polyaxon/web/polyaxon.sock;
}

server {
    include polyaxon/polyaxon.base.conf;
}
"""
        assert get_main_config() == expected



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.base import get_config

SSL_REDIRECT_OPTIONS = """
server {{
    listen 80;
    return 301 https://$host$request_uri;
}}
"""


def get_redirect_config():
    return get_config(options=SSL_REDIRECT_OPTIONS if settings.SSL_ENABLED else '',
                      indent=0,
                      ssl_path=settings.SSL_PATH)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings


def _get_indent(indent):
    return settings.INDENT_CHAR * settings.INDENT_WIDTH * indent


def get_config(options, indent=0, **kwargs):
    _options = options.format(**kwargs)
    config = []
    for p in _options.split('\n'):
        config.append('{}{}'.format(_get_indent(indent), p))

    return '\n'.join(config)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx.schemas.base import get_config

OPTIONS = """
client_max_body_size        4G;
client_body_buffer_size     50m;
client_body_in_file_only clean;
sendfile on;
"""


def get_buffering_config():
    return get_config(options=OPTIONS,
                      indent=0)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.base import get_config

OPTIONS = """
location ~ /{plugin_name}/proxy/([-_.:\w]+)/(.*) {{
    {resolver}
    rewrite_log on;
    rewrite ^/{plugin_name}/proxy/([-_.:\w]+)/(.*) /{plugin_name}/proxy/$1/$2 break;
    proxy_pass http://$1:{port};
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
}}
"""  # noqa


def get_dns_config(dns_prefix=None, dns_backend=None, dns_cluster=None):
    dns_prefix = dns_prefix or settings.DNS_PREFIX
    dns_backend = dns_backend or settings.DNS_BACKEND
    dns_cluster = dns_cluster or settings.DNS_CUSTOM_CLUSTER
    if not dns_prefix:
        dns_prefix = '{}.kube-system'.format(dns_backend)
    return '{dns_prefix}.svc.{dns_cluster}'.format(dns_prefix=dns_prefix, dns_cluster=dns_cluster)


def get_resolver():
    if settings.DNS_USE_RESOLVER:
        dns_config = get_dns_config()
        return 'resolver {} valid=5s;'.format(dns_config)
    return ''


def get_plugin_location_config(name, port):
    resolver = get_resolver()
    return get_config(options=OPTIONS,
                      indent=0,
                      plugin_name=name,
                      port=port,
                      resolver=resolver)


def get_plugins_location_config():
    plugins = []

    if settings.NGINX_PLUGINS:
        for plugin, config in settings.NGINX_PLUGINS.items():
            plugins.append(get_plugin_location_config(name=plugin, port=config['port']))

    return plugins



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.base import get_config

OPTIONS = """
location /ws/ {{
    proxy_pass http://localhost:{port};
    proxy_http_version 1.1;
    proxy_set_header Upgrade $http_upgrade;
    proxy_set_header Connection "upgrade";
    proxy_set_header Origin "";
    proxy_set_header Host $http_host;
    proxy_set_header X-Real-IP $remote_addr;
    internal;
}}
"""  # noqa


def get_ws_location_config():
    return get_config(options=OPTIONS, port=settings.WS_PORT)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.buffering import get_buffering_config
from polyaxon_nginx.schemas.charset import get_charset_config
from polyaxon_nginx.schemas.error_page import get_error_page_config
from polyaxon_nginx.schemas.gzip import get_gzip_config
from polyaxon_nginx.schemas.listen import get_listen_config
from polyaxon_nginx.schemas.locations import get_locations_config
from polyaxon_nginx.schemas.logging import get_logging_config
from polyaxon_nginx.schemas.plugins import get_plugins_location_config
from polyaxon_nginx.schemas.ssl import get_ssl_config
from polyaxon_nginx.schemas.timeout import get_timeout_config
from polyaxon_nginx.schemas.uwsgi import get_uwsgi_config
from polyaxon_nginx.schemas.ws import get_ws_location_config


def get_polyaxon_config():
    config = [
        get_listen_config(),
    ]
    if settings.SSL_ENABLED:
        config.append(get_ssl_config())
    config += [
        get_logging_config(),
        get_gzip_config(),
        get_charset_config(),
        get_buffering_config(),
        get_timeout_config(),
        get_uwsgi_config(),
        get_error_page_config(),
        get_locations_config(),
        get_ws_location_config(),
    ]
    config += get_plugins_location_config()

    return '\n'.join(config)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.base import get_config

HTTP_OPTIONS = """
listen 80;
"""

SSL_OPTIONS = """
listen 443 ssl;
ssl on;
"""


def get_listen_config():
    return get_config(options=SSL_OPTIONS if settings.SSL_ENABLED else HTTP_OPTIONS,
                      indent=0)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import os
import tempfile

from unittest import TestCase

from polyaxon_nginx.generate import generate_nginx_conf


class TestGenerate(TestCase):

    def test_generate_conf(self):
        tmp_dir = tempfile.mkdtemp()
        assert os.listdir(tmp_dir) == []
        generate_nginx_conf(path=tmp_dir)
        assert set(os.listdir(tmp_dir)) == {'polyaxon.base.conf', 'polyaxon.redirect.conf'}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.base import get_config

OPTIONS = """
send_timeout {timeout};
keepalive_timeout {timeout};
uwsgi_read_timeout {timeout};
uwsgi_send_timeout {timeout};
client_header_timeout {timeout};
proxy_read_timeout {timeout};
"""


def get_timeout_config():
    return get_config(options=OPTIONS,
                      indent=0,
                      timeout=settings.TIMEOUT)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.base import get_config

OPTIONS = """
error_log /polyaxon/logs/error.log {level};
"""


def get_logging_config():
    return get_config(options=OPTIONS,
                      indent=0,
                      level=settings.LOGGING_LEVEL)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from polyaxon_nginx import settings
from polyaxon_nginx.schemas.base import get_config

STATIC_LOCATION_OPTIONS = """
location /static/ {{
    alias /polyaxon/static/;
    autoindex on;
    expires                   30d;
    add_header                Cache-Control private;
}}
"""


def get_static_location_config():
    return get_config(options=STATIC_LOCATION_OPTIONS,
                      indent=0)


TMP_LOCATION_OPTIONS = """
location /tmp/ {{
    alias                     /tmp/;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}}
"""


def get_tmp_location_config():
    return get_config(options=TMP_LOCATION_OPTIONS,
                      indent=0)


REPOS_LOCATION_OPTIONS = """
location {repos_path}/ {{
    root                      {repos_path};
    expires                   0;
    add_header                Cache-Control private;
    internal;
}}
"""


def get_repos_location_config():
    return get_config(options=REPOS_LOCATION_OPTIONS,
                      indent=0,
                      repos_path=settings.REPOS_PATH)


DATA_LOCATION_OPTIONS = """
location {data_path}/ {{
    root                      {data_path};
    expires                   0;
    add_header                Cache-Control private;
    internal;
}}
"""


def get_data_location_config():
    return get_config(options=DATA_LOCATION_OPTIONS,
                      indent=0,
                      data_path=settings.DATA_PATH)


OUTPUTS_LOCATION_OPTIONS = """
location {outputs_path}/ {{
    root                      {outputs_path};
    expires                   0;
    add_header                Cache-Control private;
    internal;
}}
"""


def get_outputs_location_config():
    return get_config(options=OUTPUTS_LOCATION_OPTIONS,
                      indent=0,
                      outputs_path=settings.OUTPUTS_PATH)


def get_locations_config():
    config = [
        get_static_location_config(),
        get_tmp_location_config(),
        get_repos_location_config(),
        get_data_location_config(),
        get_outputs_location_config()
    ]
    return '\n'.join(config)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import os

import rhea

NGINX_CONFIG_PATH = '/polyaxon/web/nginx.json'

config = rhea.Rhea.read_configs([
    rhea.ConfigSpec(NGINX_CONFIG_PATH, check_if_exists=False),
    os.environ,
])

NGINX_PLUGINS = config.get_dict_of_dicts('POLYAXON_PLUGINS',
                                         is_optional=True,
                                         default={})
WS_PORT = config.get_int('POLYAXON_WS_PORT',
                         is_optional=True,
                         default=1337)
DNS_USE_RESOLVER = config.get_boolean('POLYAXON_DNS_USE_RESOLVER',
                                      is_optional=True,
                                      default=False)

DNS_CUSTOM_CLUSTER = config.get_string('POLYAXON_DNS_CUSTOM_CLUSTER',
                                       is_optional=True,
                                       default='cluster.local')
DNS_BACKEND = config.get_string('POLYAXON_DNS_BACKEND',
                                is_optional=True,
                                default='kube-dns')
DNS_PREFIX = config.get_string('POLYAXON_DNS_PREFIX',
                               is_optional=True)

NAMESPACE = config.get_string('POLYAXON_K8S_NAMESPACE',
                              is_optional=True)

LOGGING_LEVEL = config.get_string('POLYAXON_NGINX_LOGGING_LEVEL',
                                  is_optional=True,
                                  default='warn')

TIMEOUT = config.get_int('POLYAXON_NGINX_TIMEOUT',
                         is_optional=True,
                         default=600)

SSL_PATH = config.get_string('POLYAXON_SSL_PATH',
                             is_optional=True,
                             default='/etc/ssl/polyaxon')

INDENT_CHAR = config.get_string('POLYAXON_NGINX_INDENT_CHAR',
                                is_optional=True,
                                default=' ')

INDENT_WIDTH = config.get_int('POLYAXON_NGINX_INDENT_WIDTH',
                              is_optional=True,
                              default=4)

SSL_ENABLED = config.get_boolean('POLYAXON_SSL_ENABLED',
                                 is_optional=True,
                                 default=False)

REPOS_PATH = config.get_string('POLYAXON_NGINX_REPOS_PATH',
                               is_optional=True,
                               default='/repos')

DATA_PATH = config.get_string('POLYAXON_NGINX_DATA_PATH',
                              is_optional=True,
                              default='/data')

OUTPUTS_PATH = config.get_string('POLYAXON_NGINX_OUTPUTS_PATH',
                                 is_optional=True,
                                 default='/outputs')
